@@ -8,6 +8,7 @@ pub mod interner;
 pub mod ir;
 pub mod ir_builder;
 pub mod lang_def;
+pub mod pagerank;
 pub mod printer;
 pub mod query;
 pub mod symbol;
@@ -16,10 +17,13 @@ pub mod visit;
 
 pub use context::{CompileCtxt, CompileUnit};
 pub use graph_builder::{
-    build_llmcc_graph, BlockId, BlockRelation, GraphNode, ProjectGraph, UnitGraph,
+    build_llmcc_graph, build_llmcc_graph_with_config, BlockId, BlockRelation, GraphBuildConfig,
+    GraphNode, ProjectGraph, UnitGraph,
 };
 pub use ir::HirId;
 pub use ir_builder::build_llmcc_ir;
+pub use lang_def::LanguageTrait;
+pub use pagerank::{PageRankConfig, PageRanker, RankedBlock};
 pub use paste;
 pub use printer::{print_llmcc_graph, print_llmcc_ir};
 pub use query::{GraphBlockInfo, ProjectQuery, QueryResult};
